//
//  main.m
//  DataImport
//
//  Created by John Pannell on 7/21/14.
//  Copyright (c) 2014 John Pannell. All rights reserved.
//

#import "ESPFacility.h"
#import "ESPCourse.h"
#import "ESPTee.h"
#import "ESPHole.h"

#import "ESPRealmObjects.h"

static NSManagedObjectModel *managedObjectModel()
{
    static NSManagedObjectModel *model = nil;
    if (model != nil) {
        return model;
    }
    
    NSString *path = @"DataImport";
    path = [path stringByDeletingPathExtension];
    NSURL *modelURL = [NSURL fileURLWithPath:[path stringByAppendingPathExtension:@"momd"]];
    model = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
    
    return model;
}

static NSManagedObjectContext *managedObjectContext()
{
    static NSManagedObjectContext *context = nil;
    if (context != nil) {
        return context;
    }

    @autoreleasepool {
        context = [[NSManagedObjectContext alloc] init];
        
        NSPersistentStoreCoordinator *coordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:managedObjectModel()];
        [context setPersistentStoreCoordinator:coordinator];
        
        NSString *STORE_TYPE = NSSQLiteStoreType;
        
        NSString *path = [[NSProcessInfo processInfo] arguments][0];
        path = [path stringByDeletingPathExtension];
        NSURL *url = [NSURL fileURLWithPath:[path stringByAppendingPathExtension:@"sqlite"]];
        
        NSError *error;
        NSPersistentStore *newStore = [coordinator addPersistentStoreWithType:STORE_TYPE configuration:nil URL:url options:nil error:&error];
        
        if (newStore == nil) {
            NSLog(@"Store Configuration Failure %@", ([error localizedDescription] != nil) ? [error localizedDescription] : @"Unknown Error");
        }
    }
    return context;
}

void coreDataImport()
{
    // Create the managed object context
    NSManagedObjectContext *context = managedObjectContext();
    
    // Custom code here...
    // Save the managed object context
    NSError *error = nil;
    if (![context save:&error]) {
        NSLog(@"Error while saving %@", ([error localizedDescription] != nil) ? [error localizedDescription] : @"Unknown Error");
        exit(1);
    }
    
    NSError* err = nil;
    NSString* dataPath = [[NSBundle mainBundle] pathForResource:@"facilities" ofType:@"json"];
    
    // pull in facilities
    NSArray* facilities = [NSJSONSerialization JSONObjectWithData:[NSData dataWithContentsOfFile:dataPath]
                                                          options:kNilOptions
                                                            error:&err];
    [facilities enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        ESPFacility *facilityItem = [NSEntityDescription insertNewObjectForEntityForName:@"Facility" inManagedObjectContext:context];
        
        // set properties
        facilityItem.facilityID = [obj objectForKey:@"facility_id"];
        facilityItem.name = [obj objectForKey:@"facility_name"];
        facilityItem.address = [obj objectForKey:@"address"];
        facilityItem.city = [obj objectForKey:@"city_name"];
        facilityItem.state = [obj objectForKey:@"state_province_name"];
        facilityItem.postalCode = [obj objectForKey:@"postal_code"];
        facilityItem.phone = [obj objectForKey:@"phone_number"];
        facilityItem.website = [obj objectForKey:@"website"];
        if([obj objectForKey:@"longitude"] != [NSNull null])
            facilityItem.longitude = [obj objectForKey:@"longitude"];
        if([obj objectForKey:@"latitude"] != [NSNull null])
            facilityItem.latitude = [obj objectForKey:@"latitude"];
        facilityItem.email = [obj objectForKey:@"email_address"];
        facilityItem.lastUpdated = [NSDate date];
        
        
    }];
    if (![context save:&error]) {
        NSLog(@"Whoops, couldn't save: %@", [error localizedDescription]);
    }
    // pull in the courses
    err = nil;
    dataPath = [[NSBundle mainBundle] pathForResource:@"courses" ofType:@"json"];
    NSArray* courses = [NSJSONSerialization JSONObjectWithData:[NSData dataWithContentsOfFile:dataPath]
                                                       options:kNilOptions
                                                         error:&err];
    [courses enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        ESPCourse *courseItem = [NSEntityDescription insertNewObjectForEntityForName:@"Course" inManagedObjectContext:context];
        NSError *error;
        
        // set properties
        courseItem.courseID = [obj objectForKey:@"course_id"];
        courseItem.courseName = [obj objectForKey:@"course_name"];
        courseItem.numberOfHoles = [obj objectForKey:@"holes"];
        courseItem.par = [obj objectForKey:@"par"];
        courseItem.lastUpdated = [NSDate date];
        
        // find facility this course belongs to
        NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
        NSEntityDescription *entity = [NSEntityDescription entityForName:@"Facility" inManagedObjectContext:context];
        [fetchRequest setEntity:entity];
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"facilityID LIKE[c] %@", [obj objectForKey:@"facility_id"]];
        [fetchRequest setPredicate:predicate];
        NSArray *result = [context executeFetchRequest:fetchRequest error:&error];
        if([result count] > 0){
            [[result objectAtIndex:0] addCoursesObject:courseItem];
        } else {
            NSLog(@"Orphan Course: course_id: %@ has bad facility_id: %@", [obj objectForKey:@"course_id"], [obj objectForKey:@"facility_id"]);
        }
    }];
    if (![context save:&error]) {
        NSLog(@"Whoops, couldn't save: %@", [error localizedDescription]);
    }
    
    // pull in the tees - make tees and holes
    err = nil;
    dataPath = [[NSBundle mainBundle] pathForResource:@"tees" ofType:@"json"];
    NSArray* tees = [NSJSONSerialization JSONObjectWithData:[NSData dataWithContentsOfFile:dataPath]
                                                    options:kNilOptions
                                                      error:&err];
    [tees enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        ESPTee *teeItem = [NSEntityDescription insertNewObjectForEntityForName:@"Tee" inManagedObjectContext:context];
        NSError *error;
        
        // set properties
        teeItem.name = [obj objectForKey:@"tee_name"];
        teeItem.color = [obj objectForKey:@"tee_color"];
        teeItem.dateConfirmed = [NSDate date];
        if(![[obj objectForKey:@"rating"] isEqualToString:@"N/D"])
            teeItem.rating = [NSNumber numberWithFloat:[[obj objectForKey:@"rating"] floatValue]];
        if(![[obj objectForKey:@"slope"] isEqualToString:@"N/D"])
            teeItem.slope = [NSNumber numberWithInteger:[[obj objectForKey:@"slope"] intValue]];
        if(![[obj objectForKey:@"course_par_for_tee"] isEqualToString:@"N/D"])
            teeItem.par = [NSNumber numberWithInteger:[[obj objectForKey:@"course_par_for_tee"] intValue]];
        if(![[obj objectForKey:@"total_distance"] isEqualToString:@"N/D"])
            teeItem.totalDistance = [NSNumber numberWithInteger:[[obj objectForKey:@"total_distance"] intValue]];
        teeItem.teeID = [obj objectForKey:@"tee_id"];
        
        // create holes
        for(NSUInteger i = 1; i < 19; i++){
            NSString *holeKey = [NSString stringWithFormat:@"hole%lu", (unsigned long)i];
            NSString *parKey = [NSString stringWithFormat:@"%@_par", holeKey];
            NSString *hdcpKey = [NSString stringWithFormat:@"%@_handicap", holeKey];
            
            // check for non-existant hole
            if([[obj objectForKey:holeKey] isEqualToString:@"~"])
                continue;
            
            ESPHole *holeItem = [NSEntityDescription insertNewObjectForEntityForName:@"Hole" inManagedObjectContext:context];
            holeItem.holeNumber = [NSNumber numberWithInteger:i];
            holeItem.distance = [NSNumber numberWithInteger:[[obj objectForKey:holeKey] intValue]];
            holeItem.par =[NSNumber numberWithInteger:[[obj objectForKey:parKey] intValue]];
            holeItem.handicap = [NSNumber numberWithInteger:[[obj objectForKey:hdcpKey] intValue]];
            
            [teeItem addHolesObject:holeItem];
        }
        
        // find course this tee belongs to
        NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
        NSEntityDescription *entity = [NSEntityDescription entityForName:@"Course" inManagedObjectContext:context];
        [fetchRequest setEntity:entity];
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"courseID LIKE[c] %@", [obj objectForKey:@"course_id"]];
        [fetchRequest setPredicate:predicate];
        NSArray *result = [context executeFetchRequest:fetchRequest error:&error];
        if([result count] > 0){
            [[result objectAtIndex:0] addTeesObject:teeItem];
        } else {
            NSLog(@"Orphan Tee: tee_id: %@ has bad course_id: %@", [obj objectForKey:@"tee_id"], [obj objectForKey:@"course_id"]);
        }
        
        
    }];
    if (![context save:&error]) {
        NSLog(@"Whoops, couldn't save: %@", [error localizedDescription]);
    }
    
    
    // Test listing
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Facility"
                                              inManagedObjectContext:context];
    [fetchRequest setEntity:entity];
    NSArray *fetchedObjects = [context executeFetchRequest:fetchRequest error:&error];
    for (ESPFacility *item in fetchedObjects) {
        NSMutableString *output = [[NSMutableString alloc] init];
        [output appendString:item.name];
        for(ESPCourse *course in item.courses){
            [output appendString:[NSString stringWithFormat:@"\n\t%@", course.courseName]];
            for(ESPTee *tee in course.tees){
                [output appendString:[NSString stringWithFormat:@"\n\t\t%@", tee.name]];
            }
        }
        NSLog(@"%@",output);
    }
}

void realmImport(){
    RLMRealm *realm = [RLMRealm defaultRealm];
    NSError* err = nil;
    NSString* dataPath = [[NSBundle mainBundle] pathForResource:@"facilities" ofType:@"json"];
    
    // pull in facilities
    NSArray* facilities = [NSJSONSerialization JSONObjectWithData:[NSData dataWithContentsOfFile:dataPath]
                                                          options:kNilOptions
                                                            error:&err];
    [realm beginWriteTransaction];
    [facilities enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        
        ESPRealmFacility *facilityItem = [[ESPRealmFacility alloc] init];
        
        // set properties
        facilityItem.facilityID = [obj objectForKey:@"facility_id"];
        facilityItem.name = [obj objectForKey:@"facility_name"];
        facilityItem.address = [obj objectForKey:@"address"];
        facilityItem.city = [obj objectForKey:@"city_name"];
        facilityItem.state = [obj objectForKey:@"state_province_name"];
        facilityItem.postalCode = [obj objectForKey:@"postal_code"];
        facilityItem.phone = [obj objectForKey:@"phone_number"];
        facilityItem.website = [obj objectForKey:@"website"];
        if([obj objectForKey:@"longitude"] != [NSNull null])
            facilityItem.longitude = [[obj objectForKey:@"longitude"] floatValue];
        if([obj objectForKey:@"latitude"] != [NSNull null])
            facilityItem.latitude = [[obj objectForKey:@"latitude"] floatValue];
        facilityItem.email = [obj objectForKey:@"email_address"];
        facilityItem.lastUpdated = [NSDate date];
        
        [realm addObject:facilityItem];
        NSLog(@"added facility: %@ to %@", facilityItem.name, realm.path);
        
        
    }];
    [realm commitWriteTransaction];
    
    // now courses
    dataPath = [[NSBundle mainBundle] pathForResource:@"courses" ofType:@"json"];
    NSArray* courses = [NSJSONSerialization JSONObjectWithData:[NSData dataWithContentsOfFile:dataPath]
                                                       options:kNilOptions
                                                         error:&err];
    
    [realm beginWriteTransaction];
    [courses enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        ESPRealmCourse *courseItem = [[ESPRealmCourse alloc] init];
        
        // set properties
        courseItem.courseID = [obj objectForKey:@"course_id"];
        courseItem.courseName = [obj objectForKey:@"course_name"];
        courseItem.numberOfHoles = [[obj objectForKey:@"holes"] intValue];
        courseItem.par = [[obj objectForKey:@"par"] intValue];
        courseItem.lastUpdated = [NSDate date];
        
        // find facility this course belongs to
        RLMArray *result = [ESPRealmFacility objectsWhere:@"facilityID == %@", [obj objectForKey:@"facility_id"]];
        if([result count] > 0){
            [realm addObject:courseItem];
            [courseItem setFacility:(ESPRealmFacility *)[result objectAtIndex:0]];
            [[(ESPRealmFacility *)[result objectAtIndex:0] courses] addObject:courseItem];
            NSLog(@"course: %@", courseItem.courseName);
        } else {
            NSLog(@"Orphan Course: course_id: %@ has bad facility_id: %@", [obj objectForKey:@"course_id"], [obj objectForKey:@"facility_id"]);
        }
    }];
    [realm commitWriteTransaction];
    
    // tees and holes
    // pull in the tees - make tees and holes
    err = nil;
    dataPath = [[NSBundle mainBundle] pathForResource:@"tees" ofType:@"json"];
    NSArray* tees = [NSJSONSerialization JSONObjectWithData:[NSData dataWithContentsOfFile:dataPath]
                                                    options:kNilOptions
                                                      error:&err];
    [realm beginWriteTransaction];
    [tees enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        ESPRealmTee *teeItem = [[ESPRealmTee alloc] init];
        
        // set properties
        teeItem.name = [obj objectForKey:@"tee_name"];
        teeItem.color = [obj objectForKey:@"tee_color"];
        teeItem.dateConfirmed = [NSDate date];
        if(![[obj objectForKey:@"rating"] isEqualToString:@"N/D"])
            teeItem.rating = [[obj objectForKey:@"rating"] floatValue];
        if(![[obj objectForKey:@"slope"] isEqualToString:@"N/D"])
            teeItem.slope = [[obj objectForKey:@"slope"] intValue];
        if(![[obj objectForKey:@"course_par_for_tee"] isEqualToString:@"N/D"])
            teeItem.par = [[obj objectForKey:@"course_par_for_tee"] intValue];
        if(![[obj objectForKey:@"total_distance"] isEqualToString:@"N/D"])
            teeItem.totalDistance = [[obj objectForKey:@"total_distance"] intValue];
        teeItem.teeID = [obj objectForKey:@"tee_id"];
        [realm addObject:teeItem];
        
        // create holes
        for(int i = 1; i < 19; i++){
            NSString *holeKey = [NSString stringWithFormat:@"hole%lu", (unsigned long)i];
            NSString *parKey = [NSString stringWithFormat:@"%@_par", holeKey];
            NSString *hdcpKey = [NSString stringWithFormat:@"%@_handicap", holeKey];
            
            // check for non-existant hole
            if([[obj objectForKey:holeKey] isEqualToString:@"~"])
                continue;
            
            ESPRealmHole *holeItem = [[ESPRealmHole alloc] init];
            holeItem.holeNumber = i;
            holeItem.distance = [[obj objectForKey:holeKey] intValue];
            holeItem.par = [[obj objectForKey:parKey] intValue];
            holeItem.handicap = [[obj objectForKey:hdcpKey] intValue];
            
            [realm addObject:holeItem];
            [holeItem setTee:teeItem];
            [[teeItem holes] addObject:holeItem];
            
        }
        
        // find course this tee belongs to
        RLMArray *result = [ESPRealmCourse objectsWhere:@"courseID == %@", [obj objectForKey:@"course_id"]];
        
        if([result count] > 0){
            [[(ESPRealmCourse *)[result objectAtIndex:0] tees] addObject:teeItem];
            [teeItem setCourse:(ESPRealmCourse *)[result objectAtIndex:0]];
            NSLog(@"tee: %@", teeItem.name);
        } else {
            NSLog(@"Orphan Tee: tee_id: %@ has bad course_id: %@", [obj objectForKey:@"tee_id"], [obj objectForKey:@"course_id"]);
        }
        
        
    }];
    [realm commitWriteTransaction];
}


int main(int argc, const char * argv[])
{
    @autoreleasepool {
        //coreDataImport();
        realmImport();
        return 0;
    }
}


